package de.kif.backend

import io.ktor.application.call
import io.ktor.html.respondHtml
import io.ktor.http.content.files
import io.ktor.http.content.static
import io.ktor.routing.get
import io.ktor.routing.routing
import io.ktor.server.engine.embeddedServer
import io.ktor.server.netty.Netty
import kotlinx.html.*
import java.io.File
import java.nio.file.FileSystem
import java.nio.file.FileSystems
import java.nio.file.Files
import java.nio.file.Paths

object Main {
    private fun extractWebFolder(): File {
        val destination = Files.createTempDirectory("web");

        val classPath = "web"
        val uri = this::class.java.classLoader.getResource(classPath).toURI()
        val fileSystem: FileSystem?
        val src = if (uri.scheme == "jar") {
            fileSystem = FileSystems.newFileSystem(uri, mutableMapOf<String, Any?>())
            fileSystem.getPath(classPath)
        } else {
            Paths.get(uri)
        }

        Files.walk(src).forEach {
            val name = it.toAbsolutePath().toString().let {
                it.drop(classPath.length + it.indexOf(classPath, ignoreCase = true)).dropWhile {
                    it == '/' || it == '\\'
                }
            }
            if (Files.isDirectory(it)) {
                Files.createDirectories(destination.resolve(name))
            } else {
                Files.copy(it, destination.resolve(name))
            }
        }

        return destination.toFile()
    }

    private fun getWebFolder(): File {
        val currentDir = File(".").absoluteFile.canonicalFile

        return listOf(
            "web",
            "../web"
        ).map {
            File(currentDir, it)
        }.firstOrNull { it.isDirectory }?.absoluteFile?.canonicalFile ?: extractWebFolder()
    }

    @JvmStatic
    fun main(args: Array<String>) {
        embeddedServer(Netty, port = 8080, host = "127.0.0.1") {
            val webDir = getWebFolder()

            environment.log.info("Web directory: $webDir")

            val modules = webDir.list().toList().filter {
                it.endsWith(".js") &&
                        !it.endsWith(".meta.js") &&
                        !it.endsWith("-test.js") &&
                        it != "require.min.js"
            }.joinToString(", ") { "'${it.take(it.length - 3)}'" }

            routing {
                get("/") {
                    call.respondHtml {
                        head {
                            title("KIF Portal")

                            link(href = "/static/style/style.css", type = "text/css", rel = "stylesheet")

                            script(src = "/static/require.min.js") {}

                            script {
                                +"require.config({baseUrl: '/static'});\n"
                                +("require([$modules]);\n")
                            }
                        }
                        body {}
                    }
                }
                static("/static") {
                    files(webDir)
                }
            }
        }.start(wait = true)
    }
}